<?php

namespace App\Controller;

use App\Entity\Speaker;
use App\Entity\Template;
use App\Entity\Series;
use App\Entity\Notes;

use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Uid\Uuid;

class DefaultController extends AbstractController
{
    #[Route('/', name: 'app_default')]
    public function index(EntityManagerInterface $emi): Response
    {
        $speakers = $emi->getRepository(Speaker::class)->findAll();
        $series = $emi->getRepository(Series::class)->findAll();
        $templates = $emi->getRepository(Template::class)->findAll();
        $notes = $emi->getRepository(Notes::class)->findAll();
        $uuid = Uuid::v4();

        return $this->render('default/index.html.twig', [
            'controller_name' => 'DefaultController',
            'speakers' => $speakers,
            'series' => $series,
            'templates' => $templates,
            'notes' => $notes,
            'id' => $uuid,
        ]);
    }

    #[Route('/template-editor', name: 'app_template_editor')]
    public function templateEditor(EntityManagerInterface $emi): Response
    {
        $templates = $emi->getRepository(Template::class)->findAll();
        return $this->render('default/template-editor.html.twig', [
            'templates' => $templates
        ]);
    }

    #[Route('/reference-editor', name: 'app_reference_editor')]
    public function referenceEditor(EntityManagerInterface $emi): Response
    {
        $ref_dir = dirname(dirname(__DIR__)).'/references/';
        $creeds = $this->stripPath(glob($ref_dir.'Creeds/*.md'));
        $belgic = $this->stripPath(glob($ref_dir.'Belgic/*.md'));
        $dort = $this->stripPath(glob($ref_dir.'Dort/*.md'));
        $heidelberg = $this->stripPath(glob($ref_dir.'Heidelberg/*.md'));
        $wcf = $this->stripPath(glob($ref_dir.'Westminster/Confessions/*.md'));
        $wsc = $this->stripPath(glob($ref_dir.'Westminster/Shorter Catechism/*.md'));
        $wlc = $this->stripPath(glob($ref_dir.'Westminster/Longer Catechism/*.md'));

        return $this->render('default/reference-editor.html.twig', [
            'creeds' => $creeds,
            'belgic' => $belgic,
            'heidelberg' => $heidelberg,
            'dort' => $dort,
            'wcf' => $wcf,
            'wsc' => $wsc,
            'wlc' => $wlc,
        ]);
    }

    private function stripPath(array $files): array
    {
        $ret = [];
        foreach($files as $f) {
            $ret[] = basename($f);
        }
        return $ret;
    }
}
